

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author unascribed
 * @version 1.0
 */

import java.io.*;
import java.util.*;
import IniSection;

/**
 ** Parse a microsoft-style ini file.
 **/
public class IniFile {

	private LinkedHashMap _sectionMap = new LinkedHashMap();
  private HashMap _commentTable = new HashMap();
	private static boolean _caseOn = true;
	private static String _fileName;
	private static String _filePath;
	private static final String _WHITE_SPACE_ = "$$$WHITE_SPACE";
	private static final String _COMMENT_LINE_ = "<$$$COMMENT";
	private LineNumberReader _rd = null;

	private void setCaseOn(boolean value){
		_caseOn = value;
	}

	public void displayIniFile(){
		System.out.println(this.toBuffer());
	}

	public String toString(){
		return this.toBuffer().toString();
	}

	public StringBuffer toBuffer(){
		Set sects = this.getSections();
		StringBuffer sb = new StringBuffer();
    String commentkey;
    String commentvalue;
		for( Iterator i = sects.iterator(); i.hasNext(); ) {
			boolean found_comment_flag = false;
		  IniSection s = (IniSection) i.next();

      for (Iterator ib = _commentTable.keySet().iterator(); ib.hasNext();){
        commentkey = (String) ib.next();
        commentvalue = (String) _commentTable.get(commentkey);
			  if ( s.getName().startsWith(commentvalue) || s.getName().length() < 1  ){
			    sb.append (s.getName()+"\r\n" );
					// set found_comment_flag =true here
					// break out of for loop
					found_comment_flag = true;
					break;
			  }
      } // for

			// check the flag
			if (!found_comment_flag) {
		      sb.append( "[" + s.getName() + "]" + "\r\n" );
			}

			Map m = s.getMap();

		  for( Iterator j = m.keySet().iterator(); j.hasNext(); ) {
				String key = (String) j.next();
				if (key.startsWith("$$$WHITE_SPACE")){
					sb.append(" "+"\r\n");
				}
				else{
				  sb.append(key + " = " + m.get(key)+ "\r\n" );
				}
		  } // for
		} // for
		return sb;
	}


	public void flushToFile() {

		try {
			System.out.println(this.getFilePath());
			System.out.println(this.getFileName());
      // File f = new File(this.getFilePath()+this.getFileName());
      // if (f.exists()) {
        // f.delete();
      // }
		  PrintWriter pw = new PrintWriter( new FileWriter(this.getFilePath() + this.getFileName()));
			pw.write(this.toBuffer().toString());
			pw.flush();
			pw.close();
		} catch (Exception e){
				System.out.println(e.toString());
		}
	}

	public static boolean getCaseOn(){
		return _caseOn;
	}

	public String getFileName(){
		return this._fileName;
	}

	public File setFileName (String value){
		File f = new File(value);
		this._fileName = (f.getName());
		this.setFilePath(f.getParent() + f.separatorChar);
		return f;
	}

	public String getFilePath(){
		return this._filePath;
	}

	private void setFilePath (String value){
		this._filePath = value;
	}

	private void setLineNumberRead(String filePathName){
		File f = this.setFileName(filePathName);
		try{
		   _rd = new LineNumberReader( new FileReader( f ) );
		}catch (FileNotFoundException fnf){
			try {
		    f.createNewFile();
			  _rd = new LineNumberReader( new FileReader( f ) );
			} catch (IOException e){
				  System.out.println("Error creating LineNumberReader for path > " + filePathName + " > " + e.toString());
			}
		}
	}

	private LineNumberReader getLineReader () {
		return this._rd;
	}

	// constructor
	public IniFile( String filePathName, boolean caseOn ) throws IOException {
		this.setCaseOn(caseOn);
		this.setLineNumberRead(filePathName);
	}

	public void loadFromFile () throws IOException {
		String name;
		int i = 1;
		IniSection currentSection = new IniSection( "Defaults", this.getCaseOn() );
		String s = this.getLineReader().readLine();
		while( s != null ) {
		  s = s.trim();
		  boolean found_comment_flag = false;
      for (Iterator ib = _commentTable.keySet().iterator(); ib.hasNext();){
        String commentkey = (String) ib.next();
        String commentvalue = (String) _commentTable.get(commentkey);
			  // handle comments
		    if( s.startsWith( commentvalue ) ){
			    // make it unique so we can store it
          // and anonymous so we can ignore it
				  name = "<$$$COMMENT"+ i + ">" + s;
				  currentSection = new IniSection( name, this.getCaseOn() );
				  _sectionMap.put( name, currentSection );
					found_comment_flag = true;
					break;
		    } // if
      }// for

		  if (!found_comment_flag) {
		    // handle whitespace within sections
			  if ("".equals( s )) {
			  // make it unique so we can store it
        // and anonymous so we can ignore it
				currentSection.setValue( "$$$WHITE_SPACE"+i, i+"" );
			  }
			  // handle the section tags looking for matching brackets
		    else if( s.startsWith( "[" ) && s.endsWith( "]" ) ) {
				  if (this.getCaseOn() ){
				    name = s.substring( 1, s.length()-1 ).trim().toUpperCase();
				  }// if
				  else{
				    name = s.substring( 1, s.length()-1 ).trim();
				  }// else

				  currentSection = new IniSection( name, this.getCaseOn() );
				  _sectionMap.put( name, currentSection );

		    }// else if - looking for matching brackets
        // whoops - malformed section
        else if( s.startsWith( "[" ) ){
				  throw new IOException( "syntax error at line: "  + this.getLineReader().getLineNumber() + ">" + s );
		    }
		    else {// at this point we have a line with a key + value pair
          StringTokenizer t = new StringTokenizer( s, "=");
				  try {
					  String key = t.nextToken().trim();
					  String val = t.nextToken().trim();
            System.out.println("key: >"+key+"<");
            System.out.println("val: >"+val+"<");
				    while (t.hasMoreTokens()) {
						  val += " " + t.nextToken(" ");
              System.out.println("val: >"+val+"<");
 				    }
            currentSection.setValue( key, val );
  				}// try
				  catch (Exception NoSuchElementException){
					  System.out.println( "syntax error at line: "  + this.getLineReader().getLineNumber() + ">" + s );
  				}
  		  } // else - check for key+value pair
      } // if and only if you did not find a comment

			s = this.getLineReader().readLine();
      // bump the line counter
		  i++;
		}// while s!= null
		this.getLineReader().close();
  }

    public Set getSections() {
        LinkedHashSet s = new LinkedHashSet();
        s.addAll( _sectionMap.values() );
        return s;
    }

    public IniSection getSection( String name ) {
        return (IniSection) _sectionMap.get( name );
    }

    public void addComment(String name, String value){
      _commentTable.put(name,value);

    }

		public IniSection addSection(String value){
			if (this.hasSection(value)){
				return this.getSection(value);
		  }
		  else{
			  IniSection _sect = new IniSection(value, this.getCaseOn());
				_sectionMap.put( value, _sect );
				return _sect;
		  }
		}



		public void deleteSection(String value){
			_sectionMap.remove(value);
		}

		public boolean hasSection(String value){
			if ( this._sectionMap.containsKey(value) ){
				return true;
			}
			else {
			  return false;
			}
		}








	// just for testing
	public static void main( String[] args ) throws Exception {
		System.out.println( "--- test ---" );
		System.out.println( "  " );
		IniFile inf = new IniFile(args[0],false);


    inf.addComment("POUND","#");
    inf.addComment("SEMICOLON",";");
		// inf.addComment("DOUBLESLASH","//");

		inf.loadFromFile();
		IniSection sect = inf.addSection("fruit");
    sect.setValue("mark", "lori");
    sect.setValue("mark", "lori");
    sect.setValue("mark", "lori");
    sect.setValue("mark", "mark");
    sect.setValue("lori", "lori");
    inf.deleteSection("superbowl");




    inf.setFileName("c:\\mega.ini");

		inf.displayIniFile();
		inf.flushToFile();

	}



}


