// This is a small example phonebook application
package phonebook;

// Imports java.io for stream handling
import java.io.*;

// This class is a simple data entry for the phonebook application
class Contact {
  String m_strName;
  long m_nNumber;
}

// The application class
class PhoneBook {
  // main function creates an instance of PhoneBook and then calls the
  // main loop of the same.
  public static void main(String [] args) {
    System.out.println("Welcome to the phonebook example");
    (new PhoneBook()).doMainLoop();
  }

  // This method is a main loop that infinitely calls the dispatchCommand method
  public void doMainLoop() {
    // Iterate until IO Exception
    while (dispatchCommand())
      ;
  }

  // The dispatchCommand method will get the next token and dispatch a call to
  // a corresponding function or report an error.
  private boolean dispatchCommand() {
    System.out.print("> ");
    try {
      if (m_tokenizer.nextToken() == StreamTokenizer.TT_WORD) {
        // Check what command this was...
        String strToken = m_tokenizer.sval;
        if (strToken.equalsIgnoreCase("exit"))
          return false; // Stop main loop
        else if (strToken.equalsIgnoreCase("add"))
          addContact();
        else if (strToken.equalsIgnoreCase("find"))
          findContact();
        else
          System.out.println(strToken + ": No such command");
      } else
        System.out.println("Bad command");
    } catch (java.io.IOException e) {
      return false; // Stop main loop
    }
    return true; // True indicates we shall continue main loop
  }

  // Ask for a name and find the particular contact, or return null
  public Contact findContact() {
    try {
      // Ask for contact name
      System.out.print("Who? ");
      while (m_tokenizer.nextToken() != StreamTokenizer.TT_WORD)
        System.out.print("Please enter a name >");
      String strName = m_tokenizer.sval;
      // List any matching contacts
      int nFound = 0;
      for (int i = 0; i < m_nContactCount; ++i) {
        Contact contact = m_vecContacts[i];
        if (false == contact.m_strName.equalsIgnoreCase(strName))
          continue;
        System.out.println(contact.m_nNumber);
        ++nFound;
      }
      // Print an error if no matching contact was found
      if (0 == nFound)
        System.out.println(strName + " not found");
    } catch (IOException e) {
    }
    return null;
  }

  // This method will ask for a name and a number for a new contact.
  public Contact addContact()
  {
    try {
      // Ask for contact name
      Contact contact = new Contact();
      System.out.print("Name? ");
      while (m_tokenizer.nextToken() != StreamTokenizer.TT_WORD)
        System.out.println("Please enter a name");
      contact.m_strName = m_tokenizer.sval;
      // Ask for contact number
      System.out.print("Number? ");
      while (m_tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
        System.out.println("Please enter a number");
      contact.m_nNumber = (long)m_tokenizer.nval;
      // Insert in array
      m_vecContacts[m_nContactCount++] = contact;
      return contact;
    } catch (IOException e) {
    }
    return null;
  }

  // constructor initializes stream reader and tokenizer
  PhoneBook() {
    m_reader = new InputStreamReader(System.in);
    m_tokenizer = new java.io.StreamTokenizer(m_reader);
  }

  // Phonebook data
  private int m_nMaxContacts = 100;
  private Contact [] m_vecContacts = new Contact[m_nMaxContacts];
  private int m_nContactCount = 0;

  // Input stream processing
  InputStreamReader m_reader;
  StreamTokenizer m_tokenizer;
}
