package CustomTags.DynamicProxy;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Proxy;
import Policy.*;

// Transparent proxy around any JavaBean to handle policies.
public class BeanProxy implements InvocationHandler {

  Object[] _targets;
  String _beanClassName;
  Policy _policy;

  public BeanProxy(Object[] targets, 
                   String beanClassName,
                   Policy policy) {
    _targets = targets;
    _beanClassName = beanClassName;
    _policy = policy;
  }

  // New JavaBean and proxy instance.
  public static Object create (String beanClassName,
                               Policy policy) throws Exception {

    Class beanClass = Class.forName( beanClassName );
    Class beanIfaceClass = Class.forName( beanClassName + "Iface" );
    Object beanObj = beanClass.newInstance();

    Class [] ifaces  = {beanIfaceClass};
    Object [] targets = {beanObj};

    return Proxy.newProxyInstance(BeanProxy.class.getClassLoader(),
                             ifaces,
                             new BeanProxy(targets, beanClassName, policy));
  }

  // All bean methods are invoked through this method.
  public Object invoke(Object proxy, Method method, Object[] args)
                                                 throws Exception {
    Object result;
    _policy.execute(_beanClassName, method.getName(), 
                    "before", "log", args);
    result = method.invoke(_targets[0], args);
    _policy.execute(_beanClassName, method.getName(), 
                    "after", "log", args);
    return result;
  }

}
