package Policy;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;

// Dynamic policy handler.
public class Policy {

  Document policy = null;

  // Parse policy config file and build a DOM tree.
  public Policy (String path) throws Exception {
    File policyFile = new File( path );
    DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
    DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
    policy = docBuilder.parse(policyFile);
    return;
  }

  public void execute(String className, String methodName,
                      String position, String action, 
                      Object [] args) throws Exception {
    if ( policy == null ) {
      throw new Exception("policy error!");
    }
    Element bean = null, method = null;
    // Get all "bean" elements
    NodeList beans = policy.getElementsByTagName("bean");
    // Iterate and find matches for input parameter "className".
    for (int i=0; i<beans.getLength(); i++) {
      Element e = (Element) beans.item(i);
      if ( className.equals(e.getAttribute("className")) ) {
        bean = e;
        break;
      }
    }
    if ( bean == null ) {
      return;
    }
    // Get all "method" elements
    NodeList methods = bean.getElementsByTagName("method");
    // Iterate and find matches for input parameters
    // "methodName" and "position".
    for (int i=0; i<methods.getLength(); i++) {
      Element e = (Element) methods.item(i);
      if ( methodName.equals(e.getAttribute("name")) &&
           position.equals(e.getAttribute("position")) ) {
        method = e;
        break;
      }
    }
    if ( method == null ) {
      return;
    }
    // Act according to "action" attribute.
    if ( action.equals(method.getAttribute("action")) ) {
      String output = (new Date()).toString() +
                      " called " + className + "." + methodName +
                      " arguments ";
      if ( args == null ) {
        output = output + "NULL";
      } else {
        for (int i=0; i<args.length; i++) {
          output = output + args[i].toString();
        }
      }
      System.out.println(output);
    }
    return;
  }

}
