package net.elucidsoft.encruft;

import java.io.File;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.util.List;
import java.util.LinkedList;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.table.TableModel;
import javax.swing.JScrollPane;
import java.awt.Event;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowListener;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;

/**
 * This is the code for the App part of the enCruft demo.
 *
 * @author Shawn Garbett
 * @version 0.1
 *
 */
public class App extends JFrame
                 implements ActionListener, WindowListener, TableModelListener
{

  // Menu Command Stimuli
  private final static String NEW    = "new";
  private final static String OPEN   = "open";
  private final static String SAVE   = "save";
  private final static String REVERT = "revert";
  private final static String RENAME = "rename";
  private final static String ABOUT  = "about";

  /**
   * This is the entry point for the enCruft demo.<br/>
   * Constructs an object of type App and displays it.<br/>
   */
  public static void main(String[] args)
  {
    App encruft = new App();

    encruft.setVisible(true);
  }

 /* public App(App old)
  {
    super("enCruft");

  } */

  /**
   * Constructor. Handles the &quot;IN/Initialize&quot; Stimulus and is in
   * essence the &quot;Init&quot; Response;
   */
  public App ()
  {
    super("enCruft");


    chooser  = new JFileChooser();
    doc      = new Document();
    view     = new View(doc);
    tmpdata  = null;
    file     = null;

    JScrollPane scrollpane = new JScrollPane(view);

    getContentPane().add(scrollpane, "Center");

    // File Menu
    JMenu menu = new JMenu("File");
    menu.setMnemonic('F');
    menu.add(menuItem("Open...",             this, OPEN,   'O', KeyEvent.VK_O));
    menu.add(menuItem("New",                 this, NEW,    'N', KeyEvent.VK_N));
    menu.add(menuItem("Save",       this, SAVE,   'S', KeyEvent.VK_S));
    menu.add(revertMenu=menuItem("Revert",   this, REVERT, 'V', KeyEvent.VK_V));
    menu.add(renameMenu=menuItem("Rename...",this, RENAME, 'R', KeyEvent.VK_R));

    revertMenu.setEnabled(false);
    renameMenu.setEnabled(false);

    view.setEnabled(false);

    // Help Menu
    JMenu help = new JMenu("Help");
    help.setMnemonic('H');
    help.add(menuItem("About...", this, ABOUT,  'A', KeyEvent.VK_A)); 

    // Menu Bar  
    JMenuBar menubar = new JMenuBar();
    menubar.add(menu);
    menubar.add(help); 

    // Window Event listener
    addWindowListener(this); 
    doc.addTableModelListener(this);

    // Final setup
    setJMenuBar(menubar); 
    setSize(200, 300);

    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
  }

  /** 
   * Helper function to create menuItems
   */
  protected static JMenuItem menuItem(
      String         label,
      ActionListener listener,
      String         command,
      int            mnemonic,
      int            acceleratorKey)
  {
    JMenuItem item = new JMenuItem(label);
    item.addActionListener(listener);
    item.setActionCommand(command);
    if (mnemonic != 0)
    {
      item.setMnemonic((char)mnemonic);
    }
    if (acceleratorKey != 0)
    {
      item.setAccelerator(
          KeyStroke.getKeyStroke(
              acceleratorKey,
              Event.CTRL_MASK));
    }

    return item;
  }

  /**
   * Save document response
   */
  protected void save() throws java.io.IOException
  {
    save(file, doc.getData());
  }

  /**
   * Helper routine to improve fan-in
   */
  protected void save(File dst, List src) throws java.io.IOException
  {
    FileOutputStream out = new FileOutputStream(dst);
    ObjectOutputStream s = new ObjectOutputStream(out);
    s.writeObject(src);
    s.flush();
    s.close();
    out.close();
  }

  /**
   * Enable response
   */
  protected void enableResponse()
  {
    view.setEnabled(true);
    revertMenu.setEnabled(true);
    renameMenu.setEnabled(true);
    enabled=true;
  }

  /**
   * Open document response
   */
  protected void open(File src) throws java.lang.Exception
  {
    //System.out.println("Open");

    FileInputStream in = new FileInputStream(src);
    ObjectInputStream s = new ObjectInputStream(in);

    tmpdata = (List)s.readObject();

    s.close();
    in.close();

    file=src;
  }

  /**
   * Create an empty document, the Empty response
   */
  protected void empty(File dst) throws java.io.IOException
  {
    tmpdata = new LinkedList();
    tmpdata.add(new Integer(0));

    save(dst, tmpdata);

    file=dst;
  }

  /**
   * Transfer data read to document response
   */
  protected void data()
  {
    doc.setData(tmpdata);
  }

  /**
   * Change document title response
   */
  protected void title()
  {
    setTitle(file.getPath());
    //System.out.println("Title "+file.getPath());
  }

  /**
   * Save Existing Edits subroutine for better fan-in.
   * Code is common between handleNew(), handleSave() and handleRename()
   */
  protected boolean saveEdits()
  {
    int result; 

    if(edited)
    {
      // Start I/O in Open Thread
      try
      {
        save();
        edited=false;
      }
      catch (Exception exception)
      {
        result=JOptionPane.showConfirmDialog(view,
            "The following error occured when saving existing edits:\n"+
            exception+"\n\n"+
            "Do you wish to continue with Operation?\n"+
            "Data loss may result",
            "ERROR",
            JOptionPane.YES_NO_OPTION);
        if(result==JOptionPane.NO_OPTION) return false;
      }
    }

    return true;
  }
  
  /**
   * Routine to process the &quot;O&quot; Open Stimulus
   */
  protected void handleOpen()
  {
    int result; 

    // Save exiting edits
    if(!saveEdits()) return;

    // Active chooser in Open Thread
    result = chooser.showOpenDialog(this);
    if(result != JFileChooser.APPROVE_OPTION) return;

    // Start I/O in Open Thread
    try {open(chooser.getSelectedFile());}
    catch (Exception exception)
    {
      JOptionPane.showMessageDialog(view,
          "An error has occured of the following form:\n"+ exception,
          "ERROR",
          JOptionPane.ERROR_MESSAGE);
      return;
    }

    // Set it up
    data();
    title();

    if(!enabled) enableResponse();

    // End of open method
  }

  /**
   * Routine to process the &quot;N&quot; New Stimulus
   */
  protected void handleNew()
  {
    int result; 

    // Save exiting edits
    if(!saveEdits()) return;

    // Active chooser in New Thread
    result = chooser.showSaveDialog(this);
    if(result != JFileChooser.APPROVE_OPTION) return;

    // Verify it doesn't exist.
    if(chooser.getSelectedFile().isFile())
    {
      result=JOptionPane.showConfirmDialog(view,
        "The file "+chooser.getSelectedFile().getName()+
        " already exists\nNew will overwrite this file with an empty one.\n"+
        "Do you wish to continue?",
        "WARNING",
        JOptionPane.YES_NO_OPTION);

      if(result==JOptionPane.NO_OPTION) return;
    }

    // Start I/O in New Thread
    try {empty(chooser.getSelectedFile());}
    catch (Exception exception)
    {
      JOptionPane.showMessageDialog(view,
          "An error has occured of the following form:\n"+ exception,
          "ERROR",
          JOptionPane.ERROR_MESSAGE);
      return;
    }

    // Set it up
    data();
    title();

    if(!enabled) enableResponse();

    // End of new thread
  }

  /**
   * Routine to process the &quot;S&quot; Save Stimulus
   */
  protected void handleSave()
  {
    if(!enabled)
    {
      handleNew();
      return;
    }

    if(edited)
    {
      // Start I/O in Open Thread
      try
      {
        save();
        edited=false;
      }
      catch (Exception exception)
      {
        JOptionPane.showMessageDialog(view,
          "An error has occured of the following form:\n"+ exception,
          "ERROR",
          JOptionPane.ERROR_MESSAGE);
      }
    }
  }


  /**
   * Routine to process the &quot;V&quot; Revert Stimulus
   */
  protected void handleRevert()
  {
    // Start I/O in Revert Thread
    try {open(file);}
    catch (Exception exception)
    {
      JOptionPane.showMessageDialog(view,
          "An error has occured of the following form:\n"+ exception,
          "ERROR",
          JOptionPane.ERROR_MESSAGE);
      return;
    }

    // Set it up
    data();
    title();
    edited=false;
  }

  /**
   * Routine to process the &quot;R&quot; Rename Stimulus
   */
  protected void handleRename()
  {
    int result;

    // Save existing edits
    if(!saveEdits()) return;
    
    // Activate Chooser
    result = chooser.showSaveDialog(this);
    // Check for FR (File Rejected Response)
    if(result != JFileChooser.APPROVE_OPTION) return;

    // Verify it doesn't exist. (Exists Response)
    if(chooser.getSelectedFile().isFile())
    {
      result=JOptionPane.showConfirmDialog(view,
        "The file "+chooser.getSelectedFile().getName()+" already exists\n"+
        "Rename will overwrite this file with the current one.\n"+
        "Do you wish to continue?",
        "WARNING",
        JOptionPane.YES_NO_OPTION);

      if(result==JOptionPane.NO_OPTION) return;
    }

    try
    {
      if(file.renameTo(chooser.getSelectedFile()))
      {
        // This had to be added after failing Coverage Test Case #1
        // See: http://forum.java.sun.com/thread.jsp?forum=54&thread=313277
        file = chooser.getSelectedFile();
      }
      else
      {
        JOptionPane.showMessageDialog(view,
          "Unable to rename file\n"+
          "File may already exist and have no write permission",
          "ERROR",
          JOptionPane.ERROR_MESSAGE);
      }
    }
    catch(Exception exception)
    {
      JOptionPane.showMessageDialog(view,
          "An error has occured of the following form:\n"+ exception,
          "ERROR",
          JOptionPane.ERROR_MESSAGE);
    }
  }
   

  /**
   * This catches action requests from the user
   * and processes them as stimuli. The following
   * Stimuli are routed for processing here: O, N, S, V, R, A<br/>
   * This routine is the supporting framework for
   * event processing, that implements the Cleanroom
   * specification
   */
  public void actionPerformed(ActionEvent e)
  {
    JMenuItem item = (JMenuItem) e.getSource();
    String    cmd  = item.getActionCommand();
    
    if(     OPEN   == cmd)
    {
      handleOpen();
    }
    else if(NEW    == cmd)
    {
      handleNew();
    }
    else if(SAVE   == cmd)
    {
      handleSave();
    }
    else if(REVERT == cmd)
    {
      handleRevert();
    }
    else if(RENAME == cmd)
    {
      handleRename();
    }
    else if(ABOUT  == cmd)
    {
      JOptionPane.showMessageDialog(view, 
          "Crufty Demo\n"+
          "Shawn Garbett <Shawn@Garbett.org>\n"+
          "Version 0.1\n"+
          "Copyright (C) 2002\n"+ 
          "December 10th, 2002\n");
    }
  }

  /**
   * Stub for WindowListener implementation
   */
  public void windowActivated(WindowEvent e) {}

  /**
   * The window has closed, exit java
   */
  public void windowClosed(WindowEvent e)
  {
    //System.out.println("windowClosed");
    System.exit(0);
  }

  /**
   * Intercept the window close request as a WindowListener<br/>
   * Handles the &quot;X&quot; Exit Stimuli
   */
  public void windowClosing(WindowEvent e)
  {
    //System.out.println("windowClosing");
    if(edited)
    {
      // Start I/O in Open Thread
      try
      {
        save();
      }
      catch (Exception exception)
      {
        int result=JOptionPane.showConfirmDialog(view,
              "The following error occured when saving existing edits:\n"+
              exception+"\n\n"+
              "Do you wish to continue with Exit?\n"+
              "Data loss may result",
              "ERROR",
              JOptionPane.YES_NO_OPTION);
        if(result==JOptionPane.NO_OPTION)
        {
          //setVisible(true);
          //show();
          return; 
        }
      }
    }

    // Bye bye, 
    dispose();
  }
  
  /**
   * Stub for WindowListener implementation
   */
  public void windowDeactivated(WindowEvent e) {}

  /**
   * Stub for WindowListener implementation
   */
  public void windowDeiconified(WindowEvent e) {}

  /**
   * Stub for WindowListener implementation
   */
  public void windowIconified(WindowEvent e) {}

  /**
   * Stub for WindowListener implementation
   */
  public void windowOpened(WindowEvent e) {}

  /**
   * Handles the &quot;E&quot; Edit Stimulus
   */
  public void tableChanged(TableModelEvent e)
  {
    /*System.out.println("App::tableChanged "+
                       e.getColumn()+ " "+
                       e.getFirstRow() + " " +
                       e.getLastRow() + " " +
                       e.getType() ); */

    // Was it a result of reseting the documents data?
    if(e.getType() == TableModelEvent.UPDATE &&
       e.getColumn() == -1)
    {
      return;
    } 

    edited=true;
  }

  /**
   * State Variable Enabled, whether or not file editing is enabled
   */
  protected boolean        enabled;
  /**
   * State Variable denoted when a file has been edited and changes
   * have not been saved.
   */
  protected boolean        edited;

  // Referenced objects/state machines
  protected View           view;
  protected Document       doc;
  protected List           tmpdata;
  protected File           file;
  protected JMenuItem      revertMenu;
  protected JMenuItem      renameMenu;
  protected JFileChooser   chooser;
}

