package net.elucidsoft.encruft;

import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;

import javax.swing.table.TableCellEditor;
import javax.swing.JTable;
import javax.swing.DefaultCellEditor;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableCellRenderer;

/**
 * This is the code for the View portion of the enCruft demo.
 *
 * @author Shawn Garbett
 * @version 0.1
 *
 */
public class View extends JTable implements KeyListener
{
  /**
   * View constructor<br/>
   * It sets up a JTable view that with special number validation
   * and editing
   */
  public View(Document doc)
  {
    super(doc);
    this.doc   = doc;
  
    // Validate for numbers
    final WholeNumberField integerField = new WholeNumberField(0,9);
    integerField.setHorizontalAlignment(WholeNumberField.RIGHT);
    DefaultCellEditor integerEditor =
      new DefaultCellEditor(integerField)
          {
            public Object getCellEditorValue()
            {
                return new Integer(integerField.getValue());
            }
          };
    setDefaultEditor(Integer.class, integerEditor);  
    getSelectionModel().setSelectionMode(
        ListSelectionModel.SINGLE_SELECTION); 

    addKeyListener(this); 

    setTableHeader(null);
  }

  /**
   * This routine returns the Class of the column variables which
   * controls which cell editor gets called. Specifically added so 
   * the Integer cell editor would get called properly.
   */
  public Class getColumnClass(int c)
  {
    return doc.getValueAt(0,c).getClass();
  } 

  public String getColumnName(int c)
  {
    return "Data";
  }

  /**
   * Reference to document being viewed
   */
  protected Document doc;

  /**
   * Stub to satisfy the KeyListener interface
   */
  public void keyTyped(KeyEvent evt){}

  /**
   * Stub to satisfy the KeyListener interface
   */
  public void keyReleased(KeyEvent evt){}

  /**
   * Create new rows at the end when the enter key is pressed
   */
  public void keyPressed(KeyEvent evt)
  {
    if(evt.getKeyCode() == KeyEvent.VK_ENTER)
    {
      if(getSelectedRow() == (getRowCount()-1))
      {
        doc.add(new Integer(0));
        repaint();
      }
    }
    else if (evt.getKeyCode() == KeyEvent.VK_DELETE)
    {
      doc.remove(getSelectedRow());
    }
  } 
}

